#!/usr/bin/python

"""
Conversion from AR-IQ3 wave file to GNU Radio raw IQ files.
This file is public domain.
"""

import sys
import os
import re
import struct


def main():
    # Argument check and display how to use.
    if len(sys.argv) < 2:
        print("Usage:", sys.argv[0], "<read file>.", file=sys.stderr)
        sys.exit()

    # File exists or not.
    if not os.path.isfile(sys.argv[1]):
        print("File", sys.argv[1], "not found.")
        sys.exit()

    # Generate output file name.
    restr = "([\w\d\-\.]+)\.(WAV|wav)$"
    m = re.match(restr, sys.argv[1])
    outfilI = m.group(1) + "_transI.raw"
    outfilQ = m.group(1) + "_transQ.raw"

    # Delete the same name files.
    if os.path.isfile(outfilI):
        os.remove(outfilI)
    if os.path.isfile(outfilQ):
        os.remove(outfilQ)

    # Open file descriptor.
    fi = open(sys.argv[1], 'rb')
    fo = open(outfilI, 'wb')
    fp = open(outfilQ, 'wb')

    # First 90 bytes are ignored.
    fi.seek(90, os.SEEK_SET)

    # File Output.
    bw_I = bytearray(256)
    bw_Q = bytearray(256)

    while True:

        try :
            rdat = fi.read(512)
            rdatLen = len(rdat)
        except:
            break

        rep = rdatLen // 8

        for bpos in range(rep):
            rdat_int = struct.unpack_from('>4H', rdat, bpos*8)
            dat_I = ( (rdat_int[0] & 0xFFFE) << 16 ) | ( (rdat_int[1] & 0xFFFE) << 1 )
            struct.pack_into('I', bw_I, bpos*4, dat_I)
            dat_Q = ( (rdat_int[2] & 0xFFFE) << 16 ) | ( (rdat_int[3] & 0xFFFE) << 1 )
            struct.pack_into('I', bw_Q, bpos*4, dat_Q)

        fo.write(bw_I)
        fp.write(bw_Q)

        if rep < 64:
            break;


    # File close.
    fp.close()
    fo.close()
    fi.close()


if __name__ == '__main__':
    main()
